@@ -1,13 +1,12 @@
 package Test::Smoke::SourceTree;
 use strict;
 
-# $Id$
 use vars qw( $VERSION @EXPORT_OK %EXPORT_TAGS $NOCASE );
 $VERSION = '0.008';
 
 use File::Spec;
 use File::Find;
-use Cwd;
+use Cwd 'abs_path';
 use Carp;
 use Test::Smoke::LogMixin;
 
@@ -69,15 +68,25 @@ C<< File::Spec->rel2abs( $tree_dir) >>.
 sub new {
     my $proto = shift;
     my $class = ref $proto ? ref $proto : $proto;
+    my ($dir, $verbose) = @_;
 
     croak sprintf "Usage: my \$tree = %s->new( <directory> )", __PACKAGE__
         unless @_;
+
     # it should be a directory!
-    my $dir = File::Spec->canonpath( shift );
-    my $cwd = cwd();
+    my $cwd = abs_path(File::Spec->curdir);
     chdir $dir or croak "Cannot chdir($dir): $!";
-    my $self = {tree_dir => cwd(), verbose => shift || 0};
+    # try not to trip over an absolute-path to something that somehow is a
+    # symlink. abs_path('.') will give you the real-path; that will fail...
+    my $tree_dir = File::Spec->file_name_is_absolute($dir)
+        ? $dir
+        : abs_path(File::Spec->curdir);
+    my $self = {
+        tree_dir => $tree_dir,
+        verbose => $verbose || 0,
+    };
     chdir $cwd;
+
     return bless $self, $class;
 }
 
@@ -99,7 +108,7 @@ sub verbose { return $_[0]->{verbose} }
 
 =head2 $tree->canonpath( )
 
-C<canonpath()> returns the canonical name for the path, 
+C<canonpath()> returns the canonical name for the path,
 see L<File::Spec>.
 
 =cut
@@ -122,7 +131,7 @@ sub rel2abs {
 
 =head2 $tree->abs2rel( [$base_dir] )
 
-C<abs2rel()> returns  a relative path, 
+C<abs2rel()> returns  a relative path,
 see L<File::Spec>.
 
 =cut
@@ -134,7 +143,7 @@ sub abs2rel {
 
 =head2 $tree->mani2abs( $file[, $base_path] )
 
-C<mani2abs()> returns the absolute filename of C<$file>, which should 
+C<mani2abs()> returns the absolute filename of C<$file>, which should
 be in "MANIFEST" format (i.e. using '/' as directory separator).
 
 =cut
@@ -159,7 +168,7 @@ sub mani2abs {
 
 =head2 $tree->mani2absdir( $dir[, $base_path] )
 
-C<mani2abs()> returns the absolute dirname of C<$dir>, which should 
+C<mani2abs()> returns the absolute dirname of C<$dir>, which should
 be in "MANIFEST" format (i.e. using '/' as directory separator).
 
 =cut
@@ -180,8 +189,13 @@ C<abs2mani()> returns the MANIFEST style filename.
 
 sub abs2mani {
     my $self = shift;
-    my $relfile = File::Spec->abs2rel( File::Spec->canonpath( shift ),
-                                       $self->tree_dir );
+    my ($source_file) = @_;
+
+    my $relfile = File::Spec->abs2rel(
+        File::Spec->canonpath( $source_file ), $self->tree_dir
+    );
+    $self->log_debug("[abs2mani($source_file)] $relfile");
+
     my( undef, $directories, $file ) = File::Spec->splitpath( $relfile );
     my @dirs = grep $_ && length $_ => File::Spec->splitdir( $directories );
     push @dirs, $file;
@@ -193,7 +207,7 @@ sub abs2mani {
 C<check_MANIFEST()> reads the B<MANIFEST> file from C<< $self->tree_dir >> and
 compares it with the actual contents of C<< $self->tree_dir >>.
 
-Returns a hashref with suspicious entries (if any) as keys that have a 
+Returns a hashref with suspicious entries (if any) as keys that have a
 value of either B<ST_MISSING> (not in directory) or B<ST_UNDECLARED>
 (not in MANIFEST).
 
@@ -208,14 +222,14 @@ sub check_MANIFEST {
     my %ignore = map {
         my $entry = $NOCASE ? uc $_ : $_;
         $entry => undef
-    } ( ".patch", "MANIFEST.SKIP", '.git', '.gitignore', @_ ),
+    } ( ".patch", "MANIFEST.SKIP", '.git', '.gitignore', '.mailmap', @_ ),
       keys %{ $self->_read_mani_file( 'MANIFEST.SKIP', 1 ) };
     $self->log_debug("Found %d entries in MANIFEST.SKIP", scalar(keys %ignore));
 
     # Walk the tree, remove all found files from %manifest
-    # and add other files to %manifest 
+    # and add other files to %manifest
     # unless they are in the ignore list
-    my $cwd = cwd();
+    my $cwd = abs_path(File::Spec->curdir);
     chdir $self->tree_dir or die "Cannot chdir($self->tree_dir): $!";
     require File::Find;
     File::Find::find(
